package com.link.core.kafka.factory;

import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.kafka.KafkaProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.listener.ContainerProperties.AckMode;

@Configuration
public class KafkaConsumeFactory {
   @Autowired
   private KafkaProperties kafkaProperties;

   @Bean({"batManualContainerFactory"})
   public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, String>> batManualContainerFactory() {
      ConcurrentKafkaListenerContainerFactory<String, String> factory = this.getContainer(true, false);
      factory.getContainerProperties().setAckMode(AckMode.MANUAL_IMMEDIATE);
      return factory;
   }

   @Bean({"singleManualContainerFactory"})
   public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, String>> singleManualContainerFactory() {
      ConcurrentKafkaListenerContainerFactory<String, String> factory = this.getContainer(false, false);
      factory.getContainerProperties().setAckMode(AckMode.MANUAL_IMMEDIATE);
      return factory;
   }

   @Bean({"singleAutoContainerFactory"})
   public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, String>> singleAutoContainerFactory() {
      ConcurrentKafkaListenerContainerFactory<String, String> factory = this.getContainer(false, true);
      return factory;
   }

   private ConcurrentKafkaListenerContainerFactory<String, String> getContainer(boolean batch, boolean autoCommit) {
      ConcurrentKafkaListenerContainerFactory<String, String> factory = new ConcurrentKafkaListenerContainerFactory();
      factory.setConsumerFactory(this.consumerFactory(autoCommit));
      factory.setAutoStartup(false);
      factory.setConcurrency(this.kafkaProperties.getListener().getConcurrency());
      factory.getContainerProperties().setPollTimeout(1500L);
      factory.setBatchListener(batch);
      return factory;
   }

   public ConsumerFactory<String, String> consumerFactory(boolean autoCommit) {
      Map<String, Object> configs = this.kafkaProperties.buildConsumerProperties();
      configs.put("enable.auto.commit", autoCommit);
      return new DefaultKafkaConsumerFactory(configs);
   }
}
